package net.wanji.web.handler;

import lombok.extern.slf4j.Slf4j;
import net.wanji.feign.feignEnum.ResultEnum;
import net.wanji.feign.pojo.result.Result;
import net.wanji.web.exception.CrossException;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.servlet.http.HttpServletRequest;

/**
 * @author wuxiaokai
 * @date 2022/11/21 9:06:40
 */
@RestControllerAdvice
@Slf4j
public class GlobalExceptionHandler {

    /**
     * 处理自定义的业务异常
     */
    @ExceptionHandler(value = CrossException.class)
    public Result<String> exceptionHandler(HttpServletRequest req, CrossException e) {
        log.error("发生业务异常！原因是：{}", e.getErrorMsg());
        return Result.error(e.getErrorCode(), e.getErrorMsg());
    }

    /**
     * 处理空指针的异常
     */
    @ExceptionHandler(value = NullPointerException.class)
    public Result<String> exceptionHandler(HttpServletRequest req, NullPointerException e) {
        String errorMessage = e.getCause().getMessage();
        log.error("发生空指针异常！原因是：{}", errorMessage);
        return Result.error(ResultEnum.INTERNAL_SERVER_ERROR.getResultCode(), "发生空指针异常！原因是：" + errorMessage);
    }

    /**
     * 参数异常
     */
    @ExceptionHandler(value = MethodArgumentNotValidException.class)
    public Result<String> exceptionHandler(HttpServletRequest req, MethodArgumentNotValidException e) {
        FieldError fieldError = e.getBindingResult().getFieldError();
        String errorMessage = ResultEnum.INTERNAL_SERVER_ERROR.getResultMsg();
        if (fieldError != null) {
            errorMessage = fieldError.getDefaultMessage();
        }
        log.error("发生参数异常！原因是：{}", errorMessage);
        return Result.error(ResultEnum.INTERNAL_SERVER_ERROR.getResultCode(), "发生参数异常！原因是：" + errorMessage);
    }

    /**
     * 处理其他异常
     */
    @ExceptionHandler(value = Exception.class)
    public Result<String> exceptionHandler(HttpServletRequest req, Exception e) {
        Throwable cause = e.getCause();
        String errorMessage;
        if (cause != null) {
            errorMessage = cause.getMessage();
        } else {
            errorMessage = e.getMessage();
        }
        log.error("未知异常！原因是：{}", errorMessage);
        return Result.error(ResultEnum.INTERNAL_SERVER_ERROR.getResultCode(), "未知异常！原因是：" + errorMessage);
    }
}
